# -*- coding: utf-8 -*-

"""Test view generation."""

from .base import init_package_base_structure
from bobtemplates.plone import base
from bobtemplates.plone import view
from mrbob.bobexceptions import SkipQuestion
from mrbob.bobexceptions import ValidationError
from mrbob.configurator import Configurator
from mrbob.configurator import Question

import os
import pytest


def test_get_view_name():
    question = Question(name="view_name", question="", default=None)
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.foo.bar",
        bobconfig={"non_interactive": True},
        variables={"view_python_class": True, "view_python_class_name": "DemoView"},
    )
    view.get_view_name_from_python_class(configurator, question)
    assert question.default == "demo-view"


def test_get_view_template_name():
    question = Question(name="view_template_name", question="", default=None)
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.foo.bar",
        bobconfig={"non_interactive": True},
        variables={"view_python_class": True, "view_python_class_name": "DemoView"},
    )
    view.get_view_template_name_from_python_class(configurator, question)
    assert question.default == "demo_view"


def test_python_class_true():
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.foo.bar",
        bobconfig={"non_interactive": True},
        variables={"view_python_class": True},
    )
    view.check_python_class_answer(configurator, None)


def test_python_class_false():
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.foo.bar",
        bobconfig={"non_interactive": True},
        variables={"view_python_class": False},
    )
    with pytest.raises(SkipQuestion):
        view.check_python_class_answer(configurator, None)


def test_view_template_true():
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.foo.bar",
        bobconfig={"non_interactive": True},
        variables={"view_python_class": True, "view_template": True},
    )
    view.check_view_template_answer(configurator, None)


def test_view_template_false():
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.foo.bar",
        bobconfig={"non_interactive": True},
        variables={"view_python_class": True, "view_template": False},
    )
    with pytest.raises(SkipQuestion):
        view.check_view_template_answer(configurator, None)


def test_view_template_and_python_class_false():
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.foo.bar",
        bobconfig={"non_interactive": True},
        variables={"view_python_class": False, "view_template": False},
    )
    with pytest.raises(ValidationError):
        view.check_view_template_answer(configurator, None)


def test_update_views_configure_zcml(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    views_path = package_path + "/views/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(views_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="{{{ package.dottedname }}}">

  -*- extra stuff goes here -*-

</configure>
"""
    with open(os.path.join(views_path + "configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.sample",
        bobconfig={"non_interactive": True},
        variables={
            "view_python_class": True,
            "view_python_class_name": "MyView",
            "view_python_file_name": "py_view",
            "view_register_for": "Folder",
            "view_name": "py-view",
            "view_template": True,
            "view_template_name": "pt_view",
            "package_folder": package_path,
            "package.dottedname": "collective.sample",
            "package.browserlayer": "CollectiveSampleLayer",
        },
    )
    view._update_views_configure_zcml(configurator)

    expected = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="{{{ package.dottedname }}}">

  -*- extra stuff goes here -*-

  <browser:page
    name="py-view"
    for="plone.app.contenttypes.interfaces.IFolder"
    class=".py_view.MyView"
    template="pt_view.pt"
    permission="zope2.View"
    layer="collective.sample.interfaces.ICollectiveSampleLayer"
    />

</configure>
"""
    with open(os.path.join(views_path + "configure.zcml"), "r") as f:
        content = f.read()
        assert content == expected


def test_update_views_configure_zcml_without_template(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    views_path = package_path + "/views/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(views_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="{{{ package.dottedname }}}">

  -*- extra stuff goes here -*-

</configure>
"""
    with open(os.path.join(views_path + "configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.sample",
        bobconfig={"non_interactive": True},
        variables={
            "view_python_class": True,
            "view_python_class_name": "MyView",
            "view_python_file_name": "py_view",
            "view_register_for": "plone.app.contenttypes.interfaces.IFolder",
            "view_name": "py-view",
            "view_template": False,
            "package_folder": package_path,
            "package.dottedname": "collective.sample",
            "package.browserlayer": "CollectiveSampleLayer",
        },
    )
    view._update_views_configure_zcml(configurator)

    expected = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="{{{ package.dottedname }}}">

  -*- extra stuff goes here -*-

  <browser:page
    name="py-view"
    for="plone.app.contenttypes.interfaces.IFolder"
    class=".py_view.MyView"
    permission="zope2.View"
    layer="collective.sample.interfaces.ICollectiveSampleLayer"
    />

</configure>
"""
    with open(os.path.join(views_path + "configure.zcml"), "r") as f:
        content = f.read()
        assert content == expected


def test_update_views_configure_zcml_without_python_class(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    views_path = package_path + "/views/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(views_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="{{{ package.dottedname }}}">

  -*- extra stuff goes here -*-

</configure>
"""
    with open(os.path.join(views_path + "configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.sample",
        bobconfig={"non_interactive": True},
        variables={
            "view_python_class": False,
            "view_name": "py-view",
            "view_template": True,
            "view_template_name": "pt_view",
            "view_register_for": "*",
            "package_folder": package_path,
            "package.dottedname": "collective.sample",
            "package.browserlayer": "CollectiveSampleLayer",
        },
    )
    view._update_views_configure_zcml(configurator)

    expected = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="{{{ package.dottedname }}}">

  -*- extra stuff goes here -*-

  <browser:page
    name="py-view"
    for="*"
    template="pt_view.pt"
    permission="zope2.View"
    layer="collective.sample.interfaces.ICollectiveSampleLayer"
    />

</configure>
"""

    with open(os.path.join(views_path + "configure.zcml"), "r") as f:
        content = f.read()
        assert content == expected


def test_delete_unwanted_files_template(tmpdir):
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)
    views_path = os.path.join(package_path + "/views/")

    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={
            "view_name": "my-new-view",
            "view_python_class": False,
            "view_python_class_name": "NewView",
            "view_register_for": "*",
            "view_template": True,
            "view_template_name": "new_view",
            "plone.version": "5.1",
        },
    )
    assert configurator
    os.chdir(package_path)
    base.set_global_vars(configurator)
    configurator.render()  # pre/render/post
    # as the post_rederer also calls delete_unwanted_files. we don't need to call here
    python_file_name = configurator.variables.get("view_python_file_name") + ".py"
    template_file_name = configurator.variables.get("view_template_name") + ".pt"
    python_file_path = os.path.join(views_path + python_file_name)
    template_file_path = os.path.join(views_path + template_file_name)
    assert os.path.isfile(template_file_path)
    assert not os.path.isfile(python_file_path)


def test_delete_unwanted_files_python(tmpdir):
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)
    views_path = os.path.join(package_path + "/views/")

    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={
            "view_name": "my-new-view",
            "view_python_class": True,
            "view_python_class_name": "NewView",
            "view_register_for": "*",
            "view_template": False,
            "view_template_name": "new_view",
            "plone.version": "5.1",
        },
    )
    assert configurator
    os.chdir(package_path)
    base.set_global_vars(configurator)
    configurator.render()  # pre/render/post
    # as the post_rederer also calls delete_unwanted_files. we don't need to call here
    python_file_name = configurator.variables.get("view_python_file_name") + ".py"
    template_file_name = configurator.variables.get("view_template_name") + ".pt"
    python_file_path = os.path.join(views_path + python_file_name)
    template_file_path = os.path.join(views_path + template_file_name)
    assert not os.path.isfile(template_file_path)
    assert os.path.isfile(python_file_path)


def test_update_configure_zcml(tmpdir):
    target_path = tmpdir.strpath + "/collective.demo"
    package_path = target_path + "/src/collective/demo"
    os.makedirs(target_path)
    os.makedirs(package_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.demo">

  <i18n:registerTranslations directory="locales" />

  <!--<includeDependencies package="." />-->

  <include package=".browser" />

</configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory="collective.demo",
        bobconfig={"non_interactive": True},
        variables={"package_folder": package_path},
    )
    view._update_configure_zcml(configurator)


def test_pre_renderer(tmpdir):
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)

    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={
            "view_name": "my-new-view",
            "view_python_class": True,
            "view_python_class_name": "New_View",
            "view_base_class": "BrowserView",
            "view_template": True,
            "view_template_name": "new_view",
            "view_register_for": "plone.app.contenttypes.interfaces.IFolder",
            "plone.version": "5.1",
        },
    )

    view.prepare_renderer(configurator)

    assert configurator.variables["view_name"] == "my-new-view"
    assert configurator.variables["view_python_class_name"] == "NewView"
    assert configurator.variables["view_python_file_name"] == "new_view"

    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={
            "view_name": "my-new-view",
            "view_python_class": False,
            "view_register_for": "*",
            "view_template": False,
            "plone.version": "5.1",
        },
    )

    view.prepare_renderer(configurator)

    assert configurator.variables["view_name"] == "my-new-view"
    assert (
        configurator.variables["view_python_file_name"]
        == configurator.variables["view_name_normalized"]
    )
    assert (
        configurator.variables["view_template_name"]
        == configurator.variables["view_name_normalized"]
    )


def test_post_renderer(tmpdir):
    """Test post rendering."""
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)

    configurator = Configurator(
        template="bobtemplates.plone:view",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={
            "view_name": "my-new-view",
            "view_python_class": True,
            "view_python_class_name": "NewView",
            "view_base_class": "BrowserView",
            "view_register_for": "*",
            "view_template": True,
            "view_template_name": "new_view",
            "plone.version": "5.1",
        },
    )

    assert configurator
    os.chdir(package_path)
    base.set_global_vars(configurator)
    configurator.render()  # pre/render/post



# -*- coding: utf-8 -*-

from bobtemplates.plone.base import base_prepare_renderer
from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import remove_unwanted_files
from bobtemplates.plone.base import update_configure_zcml
from bobtemplates.plone.utils import run_black
from bobtemplates.plone.utils import run_isort
from lxml import etree

import case_conversion as cc


def get_service_name_from_python_class(configurator, question):
    """Get default service_name from python class"""
    class_name = configurator.variables["service_class_name"]
    if class_name:
        generated_name = cc.snakecase(class_name).replace("_", "-")
        question.default = generated_name
    else:
        question.default = "my-service"


def _update_package_configure_zcml(configurator):
    path = "{0}".format(
        configurator.variables["package_folder"],
    )
    file_name = "configure.zcml"
    match_xpath = "include[@package='.api']"
    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
  <include package=".api" />
"""
    update_configure_zcml(
        configurator,
        path,
        file_name=file_name,
        match_xpath=match_xpath,
        match_str=match_str,
        insert_str=insert_str,
    )


def _update_api_configure_zcml(configurator):
    path = "{0}/api".format(
        configurator.variables["package_folder"],
    )
    file_name = "configure.zcml"
    example_file_name = "{0}.example".format(file_name)
    match_xpath = "zope:include[@package='.services']"
    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
  <include package=".services" />
"""
    update_configure_zcml(
        configurator,
        path,
        file_name=file_name,
        example_file_name=example_file_name,
        match_xpath=match_xpath,
        match_str=match_str,
        insert_str=insert_str,
    )


def _update_services_configure_zcml(configurator):
    path = "{0}/api/services".format(
        configurator.variables["package_folder"],
    )
    file_name = "configure.zcml"
    example_file_name = "{0}.example".format(file_name)
    match_xpath = "zope:include[@package='.{0}']".format(
        configurator.variables["service_class_name_normalized"],
    )
    match_str = "-*- extra stuff goes here -*-"
    insert_str = '<include package=".{0}" />\n'.format(
        configurator.variables["service_class_name_normalized"],
    )
    update_configure_zcml(
        configurator,
        path,
        file_name=file_name,
        example_file_name=example_file_name,
        match_xpath=match_xpath,
        match_str=match_str,
        insert_str=insert_str,
    )


def _update_metadata_xml(configurator):
    """Add plone.restapi dependency metadata.xml in
    Generic Setup profiles.
    """
    metadata_file_name = "metadata.xml"
    metadata_file_dir = "profiles/default"
    metadata_file_path = (
        configurator.variables["package_folder"]
        + "/"
        + metadata_file_dir
        + "/"
        + metadata_file_name
    )

    with open(metadata_file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        dependencies = tree.xpath("/metadata/dependencies")[0]
        dep = "profile-plone.restapi:default"
        dep_exists = False
        for e in dependencies.iter("dependency"):
            dep_name = e.text
            if dep_name == dep:
                dep_exists = True

        if dep_exists:
            print(
                "{dep} already in metadata.xml, skip adding!".format(
                    dep=dep,
                ),
            )
            return
        dep_element = etree.Element("dependency")
        dep_element.text = dep
        dependencies.append(dep_element)

    with open(metadata_file_path, "wb") as xml_file:
        tree.write(
            xml_file,
            pretty_print=True,
            xml_declaration=True,
            encoding="utf-8",
        )


def _remove_unwanted_files(configurator):
    file_paths = []
    rel_file_paths = [
        "/api/configure.zcml.example",
        "/api/services/configure.zcml.example",
    ]
    base_path = configurator.variables["package_folder"]
    for rel_file_path in rel_file_paths:
        file_paths.append("{0}{1}".format(base_path, rel_file_path))
    remove_unwanted_files(file_paths)


def pre_renderer(configurator):
    """Pre rendering."""
    configurator = base_prepare_renderer(configurator)
    configurator.variables["template_id"] = "restapi_service"
    name = configurator.variables["service_name"].strip("_")
    name_normalized = cc.snakecase(name)
    configurator.variables["service_name_normalized"] = name_normalized
    class_name = configurator.variables["service_class_name"].strip("_")  # NOQA: E501
    configurator.variables["service_class_name"] = cc.pascalcase(  # NOQA: E501
        class_name,
    )
    configurator.variables["service_class_name_normalized"] = cc.snakecase(
        class_name,
    )
    configurator.target_directory = configurator.variables["package_folder"]


def post_renderer(configurator):
    """Post rendering."""
    _update_package_configure_zcml(configurator)
    _update_api_configure_zcml(configurator)
    _update_services_configure_zcml(configurator)
    _update_metadata_xml(configurator)
    _remove_unwanted_files(configurator)
    run_isort(configurator)
    run_black(configurator)
    git_commit(
        configurator,
        "Add restapi_service: {0}".format(
            configurator.variables["service_name"],
        ),
    )



# -*- coding: utf-8 -*-

"""Test view generation."""

from .base import init_package_base_structure
from bobtemplates.plone import base
from bobtemplates.plone import indexer
from mrbob.bobexceptions import ValidationError
from mrbob.configurator import Configurator

import os
import pytest


def test_update_indexers_configure_zcml(tmpdir):
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    indexers_path = package_path + "/indexers/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(indexers_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    i18n_domain="{{{ package.dottedname }}}">

  -*- extra stuff goes here -*-

</configure>
"""
    with open(os.path.join(indexers_path + "configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:indexer",
        target_directory="collective.sample",
        bobconfig={"non_interactive": True},
        variables={
            "indexer_name": "my_cool_index",
            "package_folder": package_path,
        },
    )
    indexer._update_indexers_configure_zcml(configurator)

    with open(os.path.join(indexers_path + "configure.zcml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_pre_renderer(tmpdir):
    base_path = tmpdir.strpath
    package_root_folder = os.path.join(
        base_path,
        "collective.foo",
    )
    package_path = init_package_base_structure(package_root_folder)
    configurator = Configurator(
        template="bobtemplates.plone:indexer",
        bobconfig={"non_interactive": True},
        target_directory=package_path,
        variables={
            "package.root_folder": package_root_folder,
            "indexer_name": "my_cool_index",
            "package_folder": os.path.join(
                package_root_folder,
                "src/collective/foo",
            ),
        },
    )
    indexer.pre_renderer(configurator)


def test_post_renderer(tmpdir):
    base_path = tmpdir.strpath
    package_root_folder = os.path.join(
        base_path,
        "collective.foo",
    )
    package_path = init_package_base_structure(package_root_folder)
    configurator = Configurator(
        template="bobtemplates.plone:indexer",
        bobconfig={"non_interactive": True},
        target_directory=package_path,
        variables={
            "package.root_folder": package_root_folder,
            "indexer_name": "my_cool_index",
            "package_folder": package_path,
        },
    )
    # os.makedirs(target_path)

    template = """
    <configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone">

    <!-- -*- extra stuff goes here -*- -->

    </configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)

    os.chdir(package_path)
    base.set_global_vars(configurator)
    configurator.render()



# -*- coding: utf-8 -*-

from bobtemplates.plone import bobregistry


def test_reg_plone_addon():
    reg = bobregistry.plone_addon()
    assert reg.template == "bobtemplates.plone:addon"
    assert reg.plonecli_alias == "addon"
    assert not reg.depend_on


def test_reg_plone_buildout():
    reg = bobregistry.plone_buildout()
    assert reg.template == "bobtemplates.plone:buildout"
    assert reg.plonecli_alias == "buildout"
    assert not reg.depend_on


def test_reg_plone_theme():
    reg = bobregistry.plone_theme()
    assert reg.template == "bobtemplates.plone:theme"
    assert reg.plonecli_alias == "theme"
    assert reg.depend_on == "plone_addon"


def test_reg_plone_content_type():
    reg = bobregistry.plone_content_type()
    assert reg.template == "bobtemplates.plone:content_type"
    assert reg.plonecli_alias == "content_type"
    assert reg.depend_on == "plone_addon"


def test_reg_plone_behavior():
    reg = bobregistry.plone_behavior()
    assert reg.template == "bobtemplates.plone:behavior"
    assert reg.plonecli_alias == "behavior"
    assert reg.depend_on == "plone_addon"


def test_reg_plone_vocabulary():
    reg = bobregistry.plone_vocabulary()
    assert reg.template == "bobtemplates.plone:vocabulary"
    assert reg.plonecli_alias == "vocabulary"
    assert reg.depend_on == "plone_addon"



# -*- coding: utf-8 -*-

from bobtemplates.plone import addon
from bobtemplates.plone import base
from mrbob.configurator import Configurator

import os


def test_pre_render(tmpdir):
    configurator = Configurator(
        template="bobtemplates.plone:addon",
        target_directory=tmpdir.strpath + "collective.foo.bar",
        variables={
            "package.dexterity_type_name": "Task",
        },
    )
    addon.pre_render(configurator)


def test_cleanup_package(tmpdir):
    target_path = tmpdir.strpath + "/collective.foo.bar"
    package_path = target_path + "/src/collective/foo/bar"
    profiles_path = package_path + "/profiles/default"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(profiles_path)
    template = """<?xml version="1.0" encoding="UTF-8"?>
<metadata>
  <version>1000</version>
  <dependencies>

  </dependencies>
</metadata>
"""
    with open(os.path.join(profiles_path + "/metadata.xml"), "w") as f:
        f.write(template)

    configurator = Configurator(
        template="bobtemplates.plone:addon",
        target_directory=target_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "package.nested": True,
            "package.namespace": "collective",
            "package.namespace2": "foo",
            "package.name": "bar",
            "year": 1970,
            "package.git.init": True,
            "package.description": "Test",
            "author.name": "The Plone Collective",
            "author.email": "collective@plone.org",
            "author.github.user": "collective",
            "plone.version": "5.1",
            "python.version": "python2.7",
            "plone.is_plone5": True,
            "vscode_support": True,
        },
    )
    assert configurator
    base.set_global_vars(configurator)
    configurator.render()


def test_no_namespace(tmpdir):
    configurator = Configurator(
        template="bobtemplates.plone:addon",
        target_directory=tmpdir.strpath + "mypackage",
        variables={},
    )
    addon.pre_render(configurator)
    name = os.path.split(tmpdir.strpath + "mypackage")[-1]
    assert configurator.variables["package.dottedname"] == name
    assert configurator.variables["package.distributionname"] == name


def test_with_dash(tmpdir):
    # no namespace
    configurator = Configurator(
        template="bobtemplates.plone:addon",
        target_directory=tmpdir.strpath + "my-package",
        variables={},
    )
    addon.pre_render(configurator)
    package_name = os.path.split(tmpdir.strpath + "my_package")[-1]
    distribution_name = os.path.split(tmpdir.strpath + "my-package")[-1]
    assert configurator.variables["package.dottedname"] == package_name
    assert configurator.variables["package.distributionname"] == distribution_name

    # with namespace
    configurator = Configurator(
        template="bobtemplates.plone:addon",
        target_directory=tmpdir.strpath + "my-namespace.my-package",
        variables={},
    )
    addon.pre_render(configurator)
    package_name = os.path.split(tmpdir.strpath + "my_namespace.my_package")[-1]
    distribution_name = os.path.split(tmpdir.strpath + "my-namespace.my-package")[-1]
    assert configurator.variables["package.dottedname"] == package_name
    assert configurator.variables["package.distributionname"] == distribution_name



# -*- coding: utf-8 -*-

"""Test form generation."""

from .base import init_package_base_structure
from bobtemplates.plone import base
from bobtemplates.plone import form
from mrbob.configurator import Configurator
from mrbob.configurator import Question

import os


def test_get_form_name():
    question = Question(name="form_name", question="", default=None)
    configurator = Configurator(
        template="bobtemplates.plone:form",
        target_directory="collective.foo.bar",
        bobconfig={"non_interactive": True},
        variables={"form_python_class_name": "FancyDemoForm"},
    )
    form.get_form_name_from_python_class(configurator, question)
    assert question.default == "fancy-demo-form"


def test_update_forms_configure_zcml(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    forms_path = package_path + "/forms/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(forms_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="{{{ package.dottedname }}}">

  -*- extra stuff goes here -*-

</configure>
"""
    with open(os.path.join(forms_path + "configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:form",
        target_directory=target_path,
        bobconfig={"non_interactive": True},
        variables={
            "form_python_file_name": "py_form",
            "form_python_class_name": "MyForm",
            "form_register_for": "Folder",
            "form_name": "py-form",
            "package_folder": package_path,
            "package.dottedname": "collective.sample",
            "package.browserlayer": "CollectiveSampleLayer",
        },
    )
    form._update_forms_configure_zcml(configurator)

    expected = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="{{{ package.dottedname }}}">

  -*- extra stuff goes here -*-

  <browser:page
    name="py-form"
    for="plone.app.contenttypes.interfaces.IFolder"
    class=".py_form.MyForm"
    permission="cmf.ManagePortal"
    layer="collective.sample.interfaces.ICollectiveSampleLayer"
    />

</configure>
"""
    with open(os.path.join(forms_path + "configure.zcml"), "r") as f:
        content = f.read()
        assert content == expected


def test_update_configure_zcml(tmpdir):
    target_path = tmpdir.strpath + "/collective.demo"
    package_path = target_path + "/src/collective/demo"
    os.makedirs(target_path)
    os.makedirs(package_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.demo">

  <i18n:registerTranslations directory="locales" />

  <!--<includeDependencies package="." />-->

  <include package=".browser" />
  <!-- -*- extra stuff goes here -*- -->

</configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:form",
        target_directory="collective.demo",
        bobconfig={"non_interactive": True},
        variables={"package_folder": package_path},
    )
    form._update_configure_zcml(configurator)


def test_pre_renderer(tmpdir):
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)

    configurator = Configurator(
        template="bobtemplates.plone:form",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={
            "form_name": "my-new-form",
            "form_python_class_name": "New_Form",
            "form_register_for": "plone.app.contenttypes.interfaces.IFolder",
            "plone.version": "6.0",
        },
    )
    form.prepare_renderer(configurator)

    assert configurator.variables["form_name"] == "my-new-form"
    assert configurator.variables["form_python_class_name"] == "NewForm"
    assert configurator.variables["form_python_file_name"] == "new_form"

    configurator = Configurator(
        template="bobtemplates.plone:form",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={
            "form_name": "my-new-form",
            "form_python_class_name": "MyNewForm",
            "form_register_for": "*",
            "plone.version": "5.1",
        },
    )

    form.prepare_renderer(configurator)

    assert configurator.variables["form_name"] == "my-new-form"
    assert (
        configurator.variables["form_python_file_name"]
        == configurator.variables["form_name_normalized"]
    )


def test_post_renderer(tmpdir):
    """Test post rendering."""
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)

    configurator = Configurator(
        template="bobtemplates.plone:form",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={
            "form_name": "my-new-form",
            "form_python_class_name": "NewForm",
            "form_register_for": "*",
            "plone.version": "5.1",
        },
    )

    assert configurator
    os.chdir(package_path)
    base.set_global_vars(configurator)
    configurator.render()  # pre/render/post



# -*- coding: utf-8 -*-

from bobtemplates.plone import base
from bobtemplates.plone import behavior
from mrbob.configurator import Configurator

import os


def test_prepare_renderer():
    configurator = Configurator(
        template="bobtemplates.plone:behavior",
        target_directory=".",
        variables={
            "behavior_name": "AttachmentType",
        },
    )
    assert configurator
    behavior.prepare_renderer(configurator)


def test_post_renderer(tmpdir):
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    profiles_path = package_path + "/profiles/default"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(profiles_path)

    template = """<?xml version="1.0" encoding="UTF-8"?>
<metadata>
  <version>1000</version>
  <dependencies>

  </dependencies>
</metadata>
"""
    with open(os.path.join(profiles_path + "/metadata.xml"), "w") as f:
        f.write(template)

    template = """
[main]
version=5.1
"""
    with open(os.path.join(target_path + "/bobtemplate.cfg"), "w") as f:
        f.write(template)

    template = """
    dummy
    '-*- Extra requirements: -*-'
"""
    with open(os.path.join(target_path + "/setup.py"), "w") as f:
        f.write(template)

    template = """
    <configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone">

    <!-- -*- extra stuff goes here -*- -->

    </configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:behavior",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "plone.version": "5.1",
            "behavior_name": "AttachmentType",
            "behavior_description": "Bla",
        },
    )

    assert configurator
    os.chdir(package_path)
    base.set_global_vars(configurator)
    behavior.prepare_renderer(configurator)
    configurator.render()
    behavior.post_renderer(configurator)



# -*- coding: utf-8 -*-
"""Generate view."""

from bobtemplates.plone.base import base_prepare_renderer
from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import update_file
from bobtemplates.plone.base import ZCML_NAMESPACES
from bobtemplates.plone.utils import run_black
from bobtemplates.plone.utils import run_isort
from lxml import etree
from mrbob.bobexceptions import SkipQuestion

import case_conversion as cc
import os


def get_view_name_from_python_class(configurator, question):
    """Generate view default name from python class"""
    view_class_name = configurator.variables["viewlet_python_class_name"]
    view_generated_name = cc.snakecase(view_class_name).replace("_", "-")  # NOQA: E501
    question.default = view_generated_name


def check_viewlet_template_answer(configurator, question):
    if not configurator.variables["viewlet_template"]:
        raise SkipQuestion(
            "No view template, so we skip view template name question."
        )  # NOQA: E501


def _update_configure_zcml(configurator):
    file_name = "configure.zcml"
    file_path = configurator.variables["package_folder"] + "/" + file_name
    namespaces = "{http://namespaces.zope.org/zope}"

    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()
        view_xpath = "{0}include[@package='.viewlets']".format(namespaces)
        if len(tree_root.findall(view_xpath)):
            print(
                ".viewlets already in configure.zcml, skip adding!",
            )
            return

    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
  <include package=".viewlets" />
"""
    update_file(configurator, file_path, match_str, insert_str)


def _update_viewlets_configure_zcml(configurator):
    file_name = "configure.zcml"
    directory_path = configurator.variables["package_folder"] + "/viewlets/"
    file_path = directory_path + file_name

    configure_example_file_path = (
        configurator.variables["package_folder"] + "/viewlets/configure.zcml.example"
    )  # NOQA: E501
    file_list = os.listdir(os.path.dirname(directory_path))
    if file_name not in file_list:
        os.rename(configure_example_file_path, file_path)

    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()
        view_xpath = "./browser:viewlet[@name='{0}']".format(
            configurator.variables["viewlet_name"],
        )
        if len(tree_root.xpath(view_xpath, namespaces=ZCML_NAMESPACES)):
            print(
                "{0} already in configure.zcml, skip adding!!!".format(
                    configurator.variables["viewlet_name"],
                ),
            )
            return

    match_str = "-*- extra stuff goes here -*-"

    iface_name = "plone.app.contenttypes.interfaces.IDocument"

    if configurator.variables["viewlet_template"]:
        insert_str = """
  <browser:viewlet
     name="{0}"
     for="{1}"
     manager="plone.app.layout.viewlets.interfaces.IAboveContentTitle"
     layer="{2}.interfaces.{3}"
     class=".{4}.{5}"
     template="{6}.pt"
     permission="zope2.View"
     />
""".format(
            configurator.variables["viewlet_name"],
            iface_name,
            configurator.variables["package.dottedname"],
            configurator.variables["browser_layer"],
            configurator.variables["viewlet_python_file_name"],
            configurator.variables["viewlet_python_class_name"],
            configurator.variables["viewlet_template_name"],
        )

    else:
        insert_str = """
  <browser:viewlet
     name="{0}"
     for="{1}"
     manager="plone.app.layout.viewlets.interfaces.IAboveContentTitle"
     layer="{2}.interfaces.{3}"
     class=".{4}.{5}"
     permission="zope2.View"
     />
""".format(
            configurator.variables["viewlet_name"],
            iface_name,
            configurator.variables["package.dottedname"],
            configurator.variables["browser_layer"],
            configurator.variables["viewlet_python_file_name"],
            configurator.variables["viewlet_python_class_name"],
        )

    update_file(configurator, file_path, match_str, insert_str)


def _delete_unwanted_files(configurator):
    directory_path = configurator.variables["package_folder"] + "/viewlets/"
    if not configurator.variables["viewlet_template"]:
        template_file_name = "{0}.pt".format(
            configurator.variables["viewlet_template_name"],
        )
        file_path = directory_path + template_file_name
        os.remove(file_path)

    file_name = "configure.zcml.example"
    file_list = os.listdir(os.path.dirname(directory_path))
    if file_name in file_list:
        file_path = directory_path + file_name
        os.remove(file_path)


def prepare_renderer(configurator):
    """Prepare rendering."""
    configurator = base_prepare_renderer(configurator)
    configurator.variables["template_id"] = "viewlet"
    viewlet_name = configurator.variables["viewlet_name"].strip("_")
    normalized_viewlet_name = cc.snakecase(viewlet_name)
    configurator.variables["viewlet_name_normalized"] = normalized_viewlet_name
    if not configurator.variables["viewlet_template"]:
        configurator.variables[
            "viewlet_template_name"
        ] = normalized_viewlet_name  # NOQA: E501
    python_class_name = configurator.variables["viewlet_python_class_name"].strip(
        "_"
    )  # NOQA: E501
    configurator.variables["viewlet_python_class_name"] = cc.pascalcase(  # NOQA: E501
        python_class_name,
    )
    viewlet_python_file_name = cc.snakecase(viewlet_name)
    configurator.variables[
        "viewlet_python_file_name"
    ] = viewlet_python_file_name  # NOQA: E501
    configurator.target_directory = configurator.variables["package_folder"]
    package_name = configurator.variables["package.dottedname"].replace(
        ".", "_"
    )  # NOQA: E501
    browser_layer = cc.pascalcase(package_name)
    configurator.variables["browser_layer"] = "I{0}Layer".format(browser_layer)


def post_renderer(configurator):
    """Post rendering."""
    _update_configure_zcml(configurator)
    _update_viewlets_configure_zcml(configurator)
    _delete_unwanted_files(configurator)
    run_isort(configurator)
    run_black(configurator)
    git_commit(
        configurator,
        "Add viewlet: {0}".format(
            configurator.variables["viewlet_name"],
        ),
    )
